@@ -1098,7 +1098,7 @@ chafa_symbol_map_find_candidates (const ChafaSymbolMap *symbol_map, guint64 bitm
 
     g_return_if_fail (symbol_map != NULL);
 
-    ham_dist = g_alloca (sizeof (gint) * (symbol_map->n_symbols + 1));
+    ham_dist = g_new (gint, symbol_map->n_symbols + 1);
 
     chafa_hamming_distance_vu64 (bitmap, symbol_map->packed_bitmaps, ham_dist, symbol_map->n_symbols);
 
@@ -1153,6 +1153,8 @@ chafa_symbol_map_find_candidates (const ChafaSymbolMap *symbol_map, guint64 bitm
 
     i = *n_candidates_inout = MIN (i, *n_candidates_inout);
     memcpy (candidates_out, candidates, i * sizeof (ChafaCandidate));
+
+    g_free (ham_dist);
 }
 
 void
@@ -1175,7 +1177,7 @@ chafa_symbol_map_find_wide_candidates (const ChafaSymbolMap *symbol_map, const g
 
     g_return_if_fail (symbol_map != NULL);
 
-    ham_dist = g_alloca (sizeof (gint) * (symbol_map->n_symbols2 + 1));
+    ham_dist = g_new (gint, symbol_map->n_symbols2 + 1);
 
     chafa_hamming_distance_2_vu64 (bitmaps, symbol_map->packed_bitmaps2, ham_dist, symbol_map->n_symbols2);
 
@@ -1230,6 +1232,8 @@ chafa_symbol_map_find_wide_candidates (const ChafaSymbolMap *symbol_map, const g
 
     i = *n_candidates_inout = MIN (i, *n_candidates_inout);
     memcpy (candidates_out, candidates, i * sizeof (ChafaCandidate));
+
+    g_free (ham_dist);
 }
 
 /* Assumes symbols are sorted by ascending popcount */
